# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, TypeAlias, TypedDict

from ..xai_model_settings_param import XaiModelSettingsParam
from ..groq_model_settings_param import GroqModelSettingsParam
from ..azure_model_settings_param import AzureModelSettingsParam
from ..text_response_format_param import TextResponseFormatParam
from ..openai_model_settings_param import OpenAIModelSettingsParam
from ..bedrock_model_settings_param import BedrockModelSettingsParam
from ..deepseek_model_settings_param import DeepseekModelSettingsParam
from ..together_model_settings_param import TogetherModelSettingsParam
from ..anthropic_model_settings_param import AnthropicModelSettingsParam
from ..google_ai_model_settings_param import GoogleAIModelSettingsParam
from ..json_object_response_format_param import JsonObjectResponseFormatParam
from ..json_schema_response_format_param import JsonSchemaResponseFormatParam
from ..google_vertex_model_settings_param import GoogleVertexModelSettingsParam

__all__ = [
    "MessageCompactParams",
    "CompactionSettings",
    "CompactionSettingsModelSettings",
    "CompactionSettingsModelSettingsZaiModelSettings",
    "CompactionSettingsModelSettingsZaiModelSettingsResponseFormat",
    "CompactionSettingsModelSettingsZaiModelSettingsThinking",
    "CompactionSettingsModelSettingsOpenRouterModelSettings",
    "CompactionSettingsModelSettingsOpenRouterModelSettingsResponseFormat",
    "CompactionSettingsModelSettingsChatGptoAuthModelSettings",
    "CompactionSettingsModelSettingsChatGptoAuthModelSettingsReasoning",
]


class MessageCompactParams(TypedDict, total=False):
    compaction_settings: Optional[CompactionSettings]
    """Configuration for conversation compaction / summarization.

    Per-model settings (temperature, max tokens, etc.) are derived from the default
    configuration for that handle.
    """


CompactionSettingsModelSettingsZaiModelSettingsResponseFormat: TypeAlias = Union[
    TextResponseFormatParam, JsonSchemaResponseFormatParam, JsonObjectResponseFormatParam
]


class CompactionSettingsModelSettingsZaiModelSettingsThinking(TypedDict, total=False):
    """The thinking configuration for GLM-4.5+ models."""

    clear_thinking: bool
    """If False, preserved thinking is used (recommended for agents)."""

    type: Literal["enabled", "disabled"]
    """Whether thinking is enabled or disabled."""


class CompactionSettingsModelSettingsZaiModelSettings(TypedDict, total=False):
    """Z.ai (ZhipuAI) model configuration (OpenAI-compatible)."""

    max_output_tokens: int
    """The maximum number of tokens the model can generate."""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calling."""

    provider_type: Literal["zai"]
    """The type of the provider."""

    response_format: Optional[CompactionSettingsModelSettingsZaiModelSettingsResponseFormat]
    """The response format for the model."""

    temperature: float
    """The temperature of the model."""

    thinking: CompactionSettingsModelSettingsZaiModelSettingsThinking
    """The thinking configuration for GLM-4.5+ models."""


CompactionSettingsModelSettingsOpenRouterModelSettingsResponseFormat: TypeAlias = Union[
    TextResponseFormatParam, JsonSchemaResponseFormatParam, JsonObjectResponseFormatParam
]


class CompactionSettingsModelSettingsOpenRouterModelSettings(TypedDict, total=False):
    """OpenRouter model configuration (OpenAI-compatible)."""

    max_output_tokens: int
    """The maximum number of tokens the model can generate."""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calling."""

    provider_type: Literal["openrouter"]
    """The type of the provider."""

    response_format: Optional[CompactionSettingsModelSettingsOpenRouterModelSettingsResponseFormat]
    """The response format for the model."""

    temperature: float
    """The temperature of the model."""


class CompactionSettingsModelSettingsChatGptoAuthModelSettingsReasoning(TypedDict, total=False):
    """The reasoning configuration for the model."""

    reasoning_effort: Literal["none", "low", "medium", "high", "xhigh"]
    """The reasoning effort level for GPT-5.x and o-series models."""


class CompactionSettingsModelSettingsChatGptoAuthModelSettings(TypedDict, total=False):
    """ChatGPT OAuth model configuration (uses ChatGPT backend API)."""

    max_output_tokens: int
    """The maximum number of tokens the model can generate."""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calling."""

    provider_type: Literal["chatgpt_oauth"]
    """The type of the provider."""

    reasoning: CompactionSettingsModelSettingsChatGptoAuthModelSettingsReasoning
    """The reasoning configuration for the model."""

    temperature: float
    """The temperature of the model."""


CompactionSettingsModelSettings: TypeAlias = Union[
    OpenAIModelSettingsParam,
    AnthropicModelSettingsParam,
    GoogleAIModelSettingsParam,
    GoogleVertexModelSettingsParam,
    AzureModelSettingsParam,
    XaiModelSettingsParam,
    CompactionSettingsModelSettingsZaiModelSettings,
    GroqModelSettingsParam,
    DeepseekModelSettingsParam,
    TogetherModelSettingsParam,
    BedrockModelSettingsParam,
    CompactionSettingsModelSettingsOpenRouterModelSettings,
    CompactionSettingsModelSettingsChatGptoAuthModelSettings,
]


class CompactionSettings(TypedDict, total=False):
    """Configuration for conversation compaction / summarization.

    Per-model settings (temperature,
    max tokens, etc.) are derived from the default configuration for that handle.
    """

    clip_chars: Optional[int]
    """The maximum length of the summary in characters.

    If none, no clipping is performed.
    """

    mode: Literal["all", "sliding_window", "self_compact_all", "self_compact_sliding_window"]
    """The type of summarization technique use."""

    model: Optional[str]
    """
    Model handle to use for sliding_window/all summarization (format:
    provider/model-name). If None, uses lightweight provider-specific defaults.
    """

    model_settings: Optional[CompactionSettingsModelSettings]
    """Optional model settings used to override defaults for the summarizer model."""

    prompt: Optional[str]
    """The prompt to use for summarization. If None, uses mode-specific default."""

    prompt_acknowledgement: bool
    """
    Whether to include an acknowledgement post-prompt (helps prevent non-summary
    outputs).
    """

    sliding_window_percentage: float
    """
    The percentage of the context window to keep post-summarization (only used in
    sliding window modes).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ToolType"]

ToolType: TypeAlias = Literal[
    "custom",
    "letta_core",
    "letta_memory_core",
    "letta_multi_agent_core",
    "letta_sleeptime_core",
    "letta_voice_sleeptime_core",
    "letta_builtin",
    "letta_files_core",
    "external_langchain",
    "external_composio",
    "external_mcp",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .message import Message
from ..._models import BaseModel
from ..stop_reason_type import StopReasonType

__all__ = [
    "LettaResponse",
    "StopReason",
    "Usage",
    "Logprobs",
    "LogprobsContent",
    "LogprobsContentTopLogprob",
    "LogprobsRefusal",
    "LogprobsRefusalTopLogprob",
    "Turn",
]


class StopReason(BaseModel):
    """The stop reason from Letta indicating why agent loop stopped execution."""

    stop_reason: StopReasonType
    """The reason why execution stopped."""

    message_type: Optional[Literal["stop_reason"]] = None
    """The type of the message."""


class Usage(BaseModel):
    """The usage statistics of the agent."""

    cache_write_tokens: Optional[int] = None
    """The number of input tokens written to cache (Anthropic only).

    None if not reported by provider.
    """

    cached_input_tokens: Optional[int] = None
    """The number of input tokens served from cache. None if not reported by provider."""

    completion_tokens: Optional[int] = None
    """The number of tokens generated by the agent."""

    context_tokens: Optional[int] = None
    """Estimate of tokens currently in the context window."""

    message_type: Optional[Literal["usage_statistics"]] = None

    prompt_tokens: Optional[int] = None
    """The number of tokens in the prompt."""

    reasoning_tokens: Optional[int] = None
    """The number of reasoning/thinking tokens generated.

    None if not reported by provider.
    """

    run_ids: Optional[List[str]] = None
    """The background task run IDs associated with the agent interaction"""

    step_count: Optional[int] = None
    """The number of steps taken by the agent."""

    total_tokens: Optional[int] = None
    """The total number of tokens processed by the agent."""


class LogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[List[int]] = None


class LogprobsContent(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[LogprobsContentTopLogprob]

    bytes: Optional[List[int]] = None


class LogprobsRefusalTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[List[int]] = None


class LogprobsRefusal(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[LogprobsRefusalTopLogprob]

    bytes: Optional[List[int]] = None


class Logprobs(BaseModel):
    """Log probabilities of the output tokens from the last LLM call.

    Only present if return_logprobs was enabled.
    """

    content: Optional[List[LogprobsContent]] = None

    refusal: Optional[List[LogprobsRefusal]] = None


class Turn(BaseModel):
    """Token data for a single LLM generation turn in a multi-turn agent interaction.

    Used for RL training to track token IDs and logprobs across all LLM calls,
    not just the final one. Tool results are included so the client can tokenize
    them with loss_mask=0 (non-trainable).
    """

    role: Literal["assistant", "tool"]
    """
    Role of this turn: 'assistant' for LLM generations (trainable), 'tool' for tool
    results (non-trainable).
    """

    content: Optional[str] = None
    """Text content. For tool turns, client tokenizes this with loss_mask=0."""

    output_ids: Optional[List[int]] = None
    """Token IDs from SGLang native endpoint. Only present for assistant turns."""

    output_token_logprobs: Optional[List[List[object]]] = None
    """Logprobs from SGLang: [[logprob, token_id, top_logprob_or_null], ...].

    Only present for assistant turns.
    """

    tool_name: Optional[str] = None
    """Name of the tool called. Only present for tool turns."""


class LettaResponse(BaseModel):
    """
    Response object from an agent interaction, consisting of the new messages generated by the agent and usage statistics.
    The type of the returned messages can be either `Message` or `LettaMessage`, depending on what was specified in the request.

    Attributes:
        messages (List[Union[Message, LettaMessage]]): The messages returned by the agent.
        usage (LettaUsageStatistics): The usage statistics
    """

    messages: List[Message]
    """The messages returned by the agent."""

    stop_reason: StopReason
    """The stop reason from Letta indicating why agent loop stopped execution."""

    usage: Usage
    """The usage statistics of the agent."""

    logprobs: Optional[Logprobs] = None
    """Log probabilities of the output tokens from the last LLM call.

    Only present if return_logprobs was enabled.
    """

    turns: Optional[List[Turn]] = None
    """Token data for all LLM generations in multi-turn agent interaction.

    Includes token IDs and logprobs for each assistant turn, plus tool result
    content. Only present if return_token_ids was enabled. Used for RL training with
    loss masking.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccessTokenListParams"]


class AccessTokenListParams(TypedDict, total=False):
    agent_id: Annotated[str, PropertyInfo(alias="agentId")]
    """The agent ID to filter tokens by.

    If provided, only tokens for this agent will be returned.
    """

    limit: float
    """The number of tokens to return per page. Defaults to 10."""

    offset: float
    """The offset for pagination. Defaults to 0."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .step import Step as Step
from .tool import Tool as Tool
from .model import Model as Model
from .folder import Folder as Folder
from .archive import Archive as Archive
from .passage import Passage as Passage
from .tool_type import ToolType as ToolType
from .agent_type import AgentType as AgentType
from .llm_config import LlmConfig as LlmConfig
from .agent_state import AgentState as AgentState
from .conversation import Conversation as Conversation
from .provider_type import ProviderType as ProviderType
from .block_response import BlockResponse as BlockResponse
from .init_tool_rule import InitToolRule as InitToolRule
from .provider_trace import ProviderTrace as ProviderTrace
from .sse_mcp_server import SseMcpServer as SseMcpServer
from .child_tool_rule import ChildToolRule as ChildToolRule
from .embedding_model import EmbeddingModel as EmbeddingModel
from .health_response import HealthResponse as HealthResponse
from .npm_requirement import NpmRequirement as NpmRequirement
from .pip_requirement import PipRequirement as PipRequirement
from .run_list_params import RunListParams as RunListParams
from .tag_list_params import TagListParams as TagListParams
from .embedding_config import EmbeddingConfig as EmbeddingConfig
from .llm_config_param import LlmConfigParam as LlmConfigParam
from .parent_tool_rule import ParentToolRule as ParentToolRule
from .stdio_mcp_server import StdioMcpServer as StdioMcpServer
from .step_list_params import StepListParams as StepListParams
from .stop_reason_type import StopReasonType as StopReasonType
from .tool_list_params import ToolListParams as ToolListParams
from .agent_list_params import AgentListParams as AgentListParams
from .block_list_params import BlockListParams as BlockListParams
from .model_list_params import ModelListParams as ModelListParams
from .provider_category import ProviderCategory as ProviderCategory
from .tag_list_response import TagListResponse as TagListResponse
from .continue_tool_rule import ContinueToolRule as ContinueToolRule
from .create_block_param import CreateBlockParam as CreateBlockParam
from .folder_list_params import FolderListParams as FolderListParams
from .terminal_tool_rule import TerminalToolRule as TerminalToolRule
from .tool_create_params import ToolCreateParams as ToolCreateParams
from .tool_search_params import ToolSearchParams as ToolSearchParams
from .tool_search_result import ToolSearchResult as ToolSearchResult
from .tool_update_params import ToolUpdateParams as ToolUpdateParams
from .tool_upsert_params import ToolUpsertParams as ToolUpsertParams
from .vector_db_provider import VectorDBProvider as VectorDBProvider
from .xai_model_settings import XaiModelSettings as XaiModelSettings
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_update_params import AgentUpdateParams as AgentUpdateParams
from .archive_list_params import ArchiveListParams as ArchiveListParams
from .block_create_params import BlockCreateParams as BlockCreateParams
from .block_update_params import BlockUpdateParams as BlockUpdateParams
from .groq_model_settings import GroqModelSettings as GroqModelSettings
from .message_list_params import MessageListParams as MessageListParams
from .model_list_response import ModelListResponse as ModelListResponse
from .tool_return_message import ToolReturnMessage as ToolReturnMessage
from .azure_model_settings import AzureModelSettings as AzureModelSettings
from .folder_create_params import FolderCreateParams as FolderCreateParams
from .folder_update_params import FolderUpdateParams as FolderUpdateParams
from .init_tool_rule_param import InitToolRuleParam as InitToolRuleParam
from .message_create_param import MessageCreateParam as MessageCreateParam
from .template_save_params import TemplateSaveParams as TemplateSaveParams
from .text_response_format import TextResponseFormat as TextResponseFormat
from .tool_search_response import ToolSearchResponse as ToolSearchResponse
from .agent_retrieve_params import AgentRetrieveParams as AgentRetrieveParams
from .archive_create_params import ArchiveCreateParams as ArchiveCreateParams
from .archive_update_params import ArchiveUpdateParams as ArchiveUpdateParams
from .child_tool_rule_param import ChildToolRuleParam as ChildToolRuleParam
from .conditional_tool_rule import ConditionalToolRule as ConditionalToolRule
from .message_list_response import MessageListResponse as MessageListResponse
from .message_search_params import MessageSearchParams as MessageSearchParams
from .npm_requirement_param import NpmRequirementParam as NpmRequirementParam
from .openai_model_settings import OpenAIModelSettings as OpenAIModelSettings
from .passage_search_params import PassageSearchParams as PassageSearchParams
from .pip_requirement_param import PipRequirementParam as PipRequirementParam
from .bedrock_model_settings import BedrockModelSettings as BedrockModelSettings
from .embedding_config_param import EmbeddingConfigParam as EmbeddingConfigParam
from .parent_tool_rule_param import ParentToolRuleParam as ParentToolRuleParam
from .template_create_params import TemplateCreateParams as TemplateCreateParams
from .template_save_response import TemplateSaveResponse as TemplateSaveResponse
from .template_update_params import TemplateUpdateParams as TemplateUpdateParams
from .deepseek_model_settings import DeepseekModelSettings as DeepseekModelSettings
from .message_search_response import MessageSearchResponse as MessageSearchResponse
from .passage_search_response import PassageSearchResponse as PassageSearchResponse
from .together_model_settings import TogetherModelSettings as TogetherModelSettings
from .access_token_list_params import AccessTokenListParams as AccessTokenListParams
from .agent_export_file_params import AgentExportFileParams as AgentExportFileParams
from .agent_import_file_params import AgentImportFileParams as AgentImportFileParams
from .anthropic_model_settings import AnthropicModelSettings as AnthropicModelSettings
from .continue_tool_rule_param import ContinueToolRuleParam as ContinueToolRuleParam
from .conversation_list_params import ConversationListParams as ConversationListParams
from .google_ai_model_settings import GoogleAIModelSettings as GoogleAIModelSettings
from .mcp_server_create_params import McpServerCreateParams as McpServerCreateParams
from .mcp_server_list_response import McpServerListResponse as McpServerListResponse
from .mcp_server_update_params import McpServerUpdateParams as McpServerUpdateParams
from .template_create_response import TemplateCreateResponse as TemplateCreateResponse
from .template_delete_response import TemplateDeleteResponse as TemplateDeleteResponse
from .template_rollback_params import TemplateRollbackParams as TemplateRollbackParams
from .template_update_response import TemplateUpdateResponse as TemplateUpdateResponse
from .terminal_tool_rule_param import TerminalToolRuleParam as TerminalToolRuleParam
from .xai_model_settings_param import XaiModelSettingsParam as XaiModelSettingsParam
from .groq_model_settings_param import GroqModelSettingsParam as GroqModelSettingsParam
from .mcp_server_refresh_params import McpServerRefreshParams as McpServerRefreshParams
from .message_retrieve_response import MessageRetrieveResponse as MessageRetrieveResponse
from .access_token_create_params import AccessTokenCreateParams as AccessTokenCreateParams
from .access_token_delete_params import AccessTokenDeleteParams as AccessTokenDeleteParams
from .access_token_list_response import AccessTokenListResponse as AccessTokenListResponse
from .agent_environment_variable import AgentEnvironmentVariable as AgentEnvironmentVariable
from .agent_export_file_response import AgentExportFileResponse as AgentExportFileResponse
from .agent_import_file_response import AgentImportFileResponse as AgentImportFileResponse
from .azure_model_settings_param import AzureModelSettingsParam as AzureModelSettingsParam
from .conversation_create_params import ConversationCreateParams as ConversationCreateParams
from .conversation_list_response import ConversationListResponse as ConversationListResponse
from .conversation_update_params import ConversationUpdateParams as ConversationUpdateParams
from .mcp_server_create_response import McpServerCreateResponse as McpServerCreateResponse
from .mcp_server_update_response import McpServerUpdateResponse as McpServerUpdateResponse
from .streamable_http_mcp_server import StreamableHTTPMcpServer as StreamableHTTPMcpServer
from .template_rollback_response import TemplateRollbackResponse as TemplateRollbackResponse
from .text_response_format_param import TextResponseFormatParam as TextResponseFormatParam
from .conditional_tool_rule_param import ConditionalToolRuleParam as ConditionalToolRuleParam
from .create_sse_mcp_server_param import CreateSseMcpServerParam as CreateSseMcpServerParam
from .json_object_response_format import JsonObjectResponseFormat as JsonObjectResponseFormat
from .json_schema_response_format import JsonSchemaResponseFormat as JsonSchemaResponseFormat
from .openai_model_settings_param import OpenAIModelSettingsParam as OpenAIModelSettingsParam
from .requires_approval_tool_rule import RequiresApprovalToolRule as RequiresApprovalToolRule
from .update_sse_mcp_server_param import UpdateSseMcpServerParam as UpdateSseMcpServerParam
from .access_token_create_response import AccessTokenCreateResponse as AccessTokenCreateResponse
from .bedrock_model_settings_param import BedrockModelSettingsParam as BedrockModelSettingsParam
from .conversation_cancel_response import ConversationCancelResponse as ConversationCancelResponse
from .google_vertex_model_settings import GoogleVertexModelSettings as GoogleVertexModelSettings
from .max_count_per_step_tool_rule import MaxCountPerStepToolRule as MaxCountPerStepToolRule
from .mcp_server_retrieve_response import McpServerRetrieveResponse as McpServerRetrieveResponse
from .create_stdio_mcp_server_param import CreateStdioMcpServerParam as CreateStdioMcpServerParam
from .deepseek_model_settings_param import DeepseekModelSettingsParam as DeepseekModelSettingsParam
from .together_model_settings_param import TogetherModelSettingsParam as TogetherModelSettingsParam
from .update_stdio_mcp_server_param import UpdateStdioMcpServerParam as UpdateStdioMcpServerParam
from .anthropic_model_settings_param import AnthropicModelSettingsParam as AnthropicModelSettingsParam
from .google_ai_model_settings_param import GoogleAIModelSettingsParam as GoogleAIModelSettingsParam
from .required_before_exit_tool_rule import RequiredBeforeExitToolRule as RequiredBeforeExitToolRule
from .json_object_response_format_param import JsonObjectResponseFormatParam as JsonObjectResponseFormatParam
from .json_schema_response_format_param import JsonSchemaResponseFormatParam as JsonSchemaResponseFormatParam
from .letta_message_content_union_param import LettaMessageContentUnionParam as LettaMessageContentUnionParam
from .requires_approval_tool_rule_param import RequiresApprovalToolRuleParam as RequiresApprovalToolRuleParam
from .google_vertex_model_settings_param import GoogleVertexModelSettingsParam as GoogleVertexModelSettingsParam
from .max_count_per_step_tool_rule_param import MaxCountPerStepToolRuleParam as MaxCountPerStepToolRuleParam
from .required_before_exit_tool_rule_param import RequiredBeforeExitToolRuleParam as RequiredBeforeExitToolRuleParam
from .create_streamable_http_mcp_server_param import (
    CreateStreamableHTTPMcpServerParam as CreateStreamableHTTPMcpServerParam,
)
from .update_streamable_http_mcp_server_param import (
    UpdateStreamableHTTPMcpServerParam as UpdateStreamableHTTPMcpServerParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import conversation_list_params, conversation_create_params, conversation_update_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.conversation import Conversation
from ...types.conversation_list_response import ConversationListResponse
from ...types.conversation_cancel_response import ConversationCancelResponse

__all__ = ["ConversationsResource", "AsyncConversationsResource"]


class ConversationsResource(SyncAPIResource):
    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConversationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return ConversationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConversationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return ConversationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        agent_id: str,
        isolated_block_labels: Optional[SequenceNotStr[str]] | Omit = omit,
        model: Optional[str] | Omit = omit,
        model_settings: Optional[conversation_create_params.ModelSettings] | Omit = omit,
        summary: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """
        Create a new conversation for an agent.

        Args:
          agent_id: The agent ID to create a conversation for

          isolated_block_labels: List of block labels that should be isolated (conversation-specific) rather than
              shared across conversations. New blocks will be created as copies of the agent's
              blocks with these labels.

          model:
              The model handle for this conversation (overrides agent's model). Format:
              provider/model-name.

          model_settings: The model settings for this conversation (overrides agent's model settings).

          summary: A summary of the conversation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/conversations/",
            body=maybe_transform(
                {
                    "isolated_block_labels": isolated_block_labels,
                    "model": model,
                    "model_settings": model_settings,
                    "summary": summary,
                },
                conversation_create_params.ConversationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"agent_id": agent_id}, conversation_create_params.ConversationCreateParams),
            ),
            cast_to=Conversation,
        )

    def retrieve(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """
        Retrieve a specific conversation.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._get(
            f"/v1/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Conversation,
        )

    def update(
        self,
        conversation_id: str,
        *,
        model: Optional[str] | Omit = omit,
        model_settings: Optional[conversation_update_params.ModelSettings] | Omit = omit,
        summary: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """Update a conversation.

        Args:
          conversation_id: The conversation identifier.

        Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          model:
              The model handle for this conversation (overrides agent's model). Format:
              provider/model-name.

          model_settings: The model settings for this conversation (overrides agent's model settings).

          summary: A summary of the conversation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._patch(
            f"/v1/conversations/{conversation_id}",
            body=maybe_transform(
                {
                    "model": model,
                    "model_settings": model_settings,
                    "summary": summary,
                },
                conversation_update_params.ConversationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Conversation,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at", "last_run_completion"] | Omit = omit,
        summary_search: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationListResponse:
        """
        List all conversations for an agent (or all conversations if agent_id not
        provided).

        Args:
          after: Cursor for pagination (conversation ID)

          agent_id: The agent ID to list conversations for (optional - returns all conversations if
              not provided)

          limit: Maximum number of conversations to return

          order: Sort order for conversations. 'asc' for oldest first, 'desc' for newest first

          order_by: Field to sort by

          summary_search: Search for text within conversation summaries

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/conversations/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                        "summary_search": summary_search,
                    },
                    conversation_list_params.ConversationListParams,
                ),
            ),
            cast_to=ConversationListResponse,
        )

    def delete(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a conversation (soft delete).

        This marks the conversation as deleted but does not permanently remove it from
        the database. The conversation will no longer appear in list operations. Any
        isolated blocks associated with the conversation will be permanently deleted.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._delete(
            f"/v1/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def cancel(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationCancelResponse:
        """
        Cancel runs associated with a conversation.

        Note: To cancel active runs, Redis is required.

        If conversation_id is an agent ID (starts with "agent-"), cancels runs for the
        agent's default conversation.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._post(
            f"/v1/conversations/{conversation_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConversationCancelResponse,
        )


class AsyncConversationsResource(AsyncAPIResource):
    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConversationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConversationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConversationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncConversationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        agent_id: str,
        isolated_block_labels: Optional[SequenceNotStr[str]] | Omit = omit,
        model: Optional[str] | Omit = omit,
        model_settings: Optional[conversation_create_params.ModelSettings] | Omit = omit,
        summary: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """
        Create a new conversation for an agent.

        Args:
          agent_id: The agent ID to create a conversation for

          isolated_block_labels: List of block labels that should be isolated (conversation-specific) rather than
              shared across conversations. New blocks will be created as copies of the agent's
              blocks with these labels.

          model:
              The model handle for this conversation (overrides agent's model). Format:
              provider/model-name.

          model_settings: The model settings for this conversation (overrides agent's model settings).

          summary: A summary of the conversation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/conversations/",
            body=await async_maybe_transform(
                {
                    "isolated_block_labels": isolated_block_labels,
                    "model": model,
                    "model_settings": model_settings,
                    "summary": summary,
                },
                conversation_create_params.ConversationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"agent_id": agent_id}, conversation_create_params.ConversationCreateParams
                ),
            ),
            cast_to=Conversation,
        )

    async def retrieve(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """
        Retrieve a specific conversation.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._get(
            f"/v1/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Conversation,
        )

    async def update(
        self,
        conversation_id: str,
        *,
        model: Optional[str] | Omit = omit,
        model_settings: Optional[conversation_update_params.ModelSettings] | Omit = omit,
        summary: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Conversation:
        """Update a conversation.

        Args:
          conversation_id: The conversation identifier.

        Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          model:
              The model handle for this conversation (overrides agent's model). Format:
              provider/model-name.

          model_settings: The model settings for this conversation (overrides agent's model settings).

          summary: A summary of the conversation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._patch(
            f"/v1/conversations/{conversation_id}",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "model_settings": model_settings,
                    "summary": summary,
                },
                conversation_update_params.ConversationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Conversation,
        )

    async def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at", "last_run_completion"] | Omit = omit,
        summary_search: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationListResponse:
        """
        List all conversations for an agent (or all conversations if agent_id not
        provided).

        Args:
          after: Cursor for pagination (conversation ID)

          agent_id: The agent ID to list conversations for (optional - returns all conversations if
              not provided)

          limit: Maximum number of conversations to return

          order: Sort order for conversations. 'asc' for oldest first, 'desc' for newest first

          order_by: Field to sort by

          summary_search: Search for text within conversation summaries

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/conversations/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                        "summary_search": summary_search,
                    },
                    conversation_list_params.ConversationListParams,
                ),
            ),
            cast_to=ConversationListResponse,
        )

    async def delete(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a conversation (soft delete).

        This marks the conversation as deleted but does not permanently remove it from
        the database. The conversation will no longer appear in list operations. Any
        isolated blocks associated with the conversation will be permanently deleted.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._delete(
            f"/v1/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def cancel(
        self,
        conversation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationCancelResponse:
        """
        Cancel runs associated with a conversation.

        Note: To cancel active runs, Redis is required.

        If conversation_id is an agent ID (starts with "agent-"), cancels runs for the
        agent's default conversation.

        Args:
          conversation_id: The conversation identifier. Can be a conversation ID ('conv-<uuid4>'), an agent
              ID ('agent-<uuid4>') for agent-direct messaging, or 'default'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._post(
            f"/v1/conversations/{conversation_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConversationCancelResponse,
        )


class ConversationsResourceWithRawResponse:
    def __init__(self, conversations: ConversationsResource) -> None:
        self._conversations = conversations

        self.create = to_raw_response_wrapper(
            conversations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            conversations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            conversations.update,
        )
        self.list = to_raw_response_wrapper(
            conversations.list,
        )
        self.delete = to_raw_response_wrapper(
            conversations.delete,
        )
        self.cancel = to_raw_response_wrapper(
            conversations.cancel,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._conversations.messages)


class AsyncConversationsResourceWithRawResponse:
    def __init__(self, conversations: AsyncConversationsResource) -> None:
        self._conversations = conversations

        self.create = async_to_raw_response_wrapper(
            conversations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            conversations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            conversations.update,
        )
        self.list = async_to_raw_response_wrapper(
            conversations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            conversations.delete,
        )
        self.cancel = async_to_raw_response_wrapper(
            conversations.cancel,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._conversations.messages)


class ConversationsResourceWithStreamingResponse:
    def __init__(self, conversations: ConversationsResource) -> None:
        self._conversations = conversations

        self.create = to_streamed_response_wrapper(
            conversations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            conversations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            conversations.update,
        )
        self.list = to_streamed_response_wrapper(
            conversations.list,
        )
        self.delete = to_streamed_response_wrapper(
            conversations.delete,
        )
        self.cancel = to_streamed_response_wrapper(
            conversations.cancel,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._conversations.messages)


class AsyncConversationsResourceWithStreamingResponse:
    def __init__(self, conversations: AsyncConversationsResource) -> None:
        self._conversations = conversations

        self.create = async_to_streamed_response_wrapper(
            conversations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            conversations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            conversations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            conversations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            conversations.delete,
        )
        self.cancel = async_to_streamed_response_wrapper(
            conversations.cancel,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._conversations.messages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .user_message import UserMessage
from .system_message import SystemMessage
from ..stop_reason_type import StopReasonType
from .assistant_message import AssistantMessage
from .reasoning_message import ReasoningMessage
from .tool_call_message import ToolCallMessage
from ..tool_return_message import ToolReturnMessage
from .approval_request_message import ApprovalRequestMessage
from .hidden_reasoning_message import HiddenReasoningMessage
from .approval_response_message import ApprovalResponseMessage

__all__ = ["LettaStreamingResponse", "LettaPing", "LettaErrorMessage", "LettaStopReason", "LettaUsageStatistics"]


class LettaPing(BaseModel):
    """
    A ping message used as a keepalive to prevent SSE streams from timing out during long running requests.

    Args:
        id (str): The ID of the message
        date (datetime): The date the message was created in ISO format
    """

    id: str

    date: datetime

    is_err: Optional[bool] = None

    message_type: Optional[Literal["ping"]] = None
    """The type of the message.

    Ping messages are a keep-alive to prevent SSE streams from timing out during
    long running requests.
    """

    name: Optional[str] = None

    otid: Optional[str] = None

    run_id: Optional[str] = None

    sender_id: Optional[str] = None

    seq_id: Optional[int] = None

    step_id: Optional[str] = None


class LettaErrorMessage(BaseModel):
    """
    Error messages are used to notify the client of an error that occurred during the agent's execution.
    """

    error_type: str
    """The type of error."""

    message: str
    """The error message."""

    message_type: Literal["error_message"]
    """The type of the message."""

    run_id: str
    """The ID of the run."""

    detail: Optional[str] = None
    """An optional error detail."""

    seq_id: Optional[int] = None
    """The sequence ID for cursor-based pagination."""


class LettaStopReason(BaseModel):
    """The stop reason from Letta indicating why agent loop stopped execution."""

    stop_reason: StopReasonType
    """The reason why execution stopped."""

    message_type: Optional[Literal["stop_reason"]] = None
    """The type of the message."""


class LettaUsageStatistics(BaseModel):
    """Usage statistics for the agent interaction.

    Attributes:
        completion_tokens (int): The number of tokens generated by the agent.
        prompt_tokens (int): The number of tokens in the prompt.
        total_tokens (int): The total number of tokens processed by the agent.
        step_count (int): The number of steps taken by the agent.
        cached_input_tokens (Optional[int]): The number of input tokens served from cache. None if not reported.
        cache_write_tokens (Optional[int]): The number of input tokens written to cache. None if not reported.
        reasoning_tokens (Optional[int]): The number of reasoning/thinking tokens generated. None if not reported.
    """

    cache_write_tokens: Optional[int] = None
    """The number of input tokens written to cache (Anthropic only).

    None if not reported by provider.
    """

    cached_input_tokens: Optional[int] = None
    """The number of input tokens served from cache. None if not reported by provider."""

    completion_tokens: Optional[int] = None
    """The number of tokens generated by the agent."""

    context_tokens: Optional[int] = None
    """Estimate of tokens currently in the context window."""

    message_type: Optional[Literal["usage_statistics"]] = None

    prompt_tokens: Optional[int] = None
    """The number of tokens in the prompt."""

    reasoning_tokens: Optional[int] = None
    """The number of reasoning/thinking tokens generated.

    None if not reported by provider.
    """

    run_ids: Optional[List[str]] = None
    """The background task run IDs associated with the agent interaction"""

    step_count: Optional[int] = None
    """The number of steps taken by the agent."""

    total_tokens: Optional[int] = None
    """The total number of tokens processed by the agent."""


LettaStreamingResponse: TypeAlias = Annotated[
    Union[
        SystemMessage,
        UserMessage,
        ReasoningMessage,
        HiddenReasoningMessage,
        ToolCallMessage,
        ToolReturnMessage,
        AssistantMessage,
        ApprovalRequestMessage,
        ApprovalResponseMessage,
        LettaPing,
        LettaErrorMessage,
        LettaStopReason,
        LettaUsageStatistics,
    ],
    PropertyInfo(discriminator="message_type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from .trace import (
    TraceResource,
    AsyncTraceResource,
    TraceResourceWithRawResponse,
    AsyncTraceResourceWithRawResponse,
    TraceResourceWithStreamingResponse,
    AsyncTraceResourceWithStreamingResponse,
)
from ...types import step_list_params
from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform
from .feedback import (
    FeedbackResource,
    AsyncFeedbackResource,
    FeedbackResourceWithRawResponse,
    AsyncFeedbackResourceWithRawResponse,
    FeedbackResourceWithStreamingResponse,
    AsyncFeedbackResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ...types.step import Step
from ..._base_client import AsyncPaginator, make_request_options

__all__ = ["StepsResource", "AsyncStepsResource"]


class StepsResource(SyncAPIResource):
    @cached_property
    def metrics(self) -> MetricsResource:
        return MetricsResource(self._client)

    @cached_property
    def trace(self) -> TraceResource:
        return TraceResource(self._client)

    @cached_property
    def feedback(self) -> FeedbackResource:
        return FeedbackResource(self._client)

    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> StepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return StepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return StepsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        step_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Get a step by ID.

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._get(
            f"/v1/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        end_date: Optional[str] | Omit = omit,
        feedback: Optional[Literal["positive", "negative"]] | Omit = omit,
        has_feedback: Optional[bool] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        model: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        start_date: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        trace_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Step]:
        """
        List steps with optional pagination and date filters.

        Args:
          after: Return steps after this step ID

          agent_id: Filter by the ID of the agent that performed the step

          before: Return steps before this step ID

          end_date: Return steps before this ISO datetime (e.g. "2025-01-29T15:01:19-08:00")

          feedback: Filter by feedback

          has_feedback: Filter by whether steps have feedback (true) or not (false)

          limit: Maximum number of steps to return

          model: Filter by the name of the model used for the step

          order: Sort order for steps by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          project_id: Filter by the project ID that is associated with the step (cloud only).

          start_date: Return steps after this ISO datetime (e.g. "2025-01-29T15:01:19-08:00")

          tags: Filter by tags

          trace_ids: Filter by trace ids returned by the server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/steps/",
            page=SyncArrayPage[Step],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "before": before,
                        "end_date": end_date,
                        "feedback": feedback,
                        "has_feedback": has_feedback,
                        "limit": limit,
                        "model": model,
                        "order": order,
                        "order_by": order_by,
                        "project_id": project_id,
                        "start_date": start_date,
                        "tags": tags,
                        "trace_ids": trace_ids,
                    },
                    step_list_params.StepListParams,
                ),
            ),
            model=Step,
        )


class AsyncStepsResource(AsyncAPIResource):
    @cached_property
    def metrics(self) -> AsyncMetricsResource:
        return AsyncMetricsResource(self._client)

    @cached_property
    def trace(self) -> AsyncTraceResource:
        return AsyncTraceResource(self._client)

    @cached_property
    def feedback(self) -> AsyncFeedbackResource:
        return AsyncFeedbackResource(self._client)

    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncStepsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        step_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Get a step by ID.

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._get(
            f"/v1/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        end_date: Optional[str] | Omit = omit,
        feedback: Optional[Literal["positive", "negative"]] | Omit = omit,
        has_feedback: Optional[bool] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        model: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        start_date: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        trace_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Step, AsyncArrayPage[Step]]:
        """
        List steps with optional pagination and date filters.

        Args:
          after: Return steps after this step ID

          agent_id: Filter by the ID of the agent that performed the step

          before: Return steps before this step ID

          end_date: Return steps before this ISO datetime (e.g. "2025-01-29T15:01:19-08:00")

          feedback: Filter by feedback

          has_feedback: Filter by whether steps have feedback (true) or not (false)

          limit: Maximum number of steps to return

          model: Filter by the name of the model used for the step

          order: Sort order for steps by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          project_id: Filter by the project ID that is associated with the step (cloud only).

          start_date: Return steps after this ISO datetime (e.g. "2025-01-29T15:01:19-08:00")

          tags: Filter by tags

          trace_ids: Filter by trace ids returned by the server

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/steps/",
            page=AsyncArrayPage[Step],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "before": before,
                        "end_date": end_date,
                        "feedback": feedback,
                        "has_feedback": has_feedback,
                        "limit": limit,
                        "model": model,
                        "order": order,
                        "order_by": order_by,
                        "project_id": project_id,
                        "start_date": start_date,
                        "tags": tags,
                        "trace_ids": trace_ids,
                    },
                    step_list_params.StepListParams,
                ),
            ),
            model=Step,
        )


class StepsResourceWithRawResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.retrieve = to_raw_response_wrapper(
            steps.retrieve,
        )
        self.list = to_raw_response_wrapper(
            steps.list,
        )

    @cached_property
    def metrics(self) -> MetricsResourceWithRawResponse:
        return MetricsResourceWithRawResponse(self._steps.metrics)

    @cached_property
    def trace(self) -> TraceResourceWithRawResponse:
        return TraceResourceWithRawResponse(self._steps.trace)

    @cached_property
    def feedback(self) -> FeedbackResourceWithRawResponse:
        return FeedbackResourceWithRawResponse(self._steps.feedback)

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._steps.messages)


class AsyncStepsResourceWithRawResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.retrieve = async_to_raw_response_wrapper(
            steps.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            steps.list,
        )

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithRawResponse:
        return AsyncMetricsResourceWithRawResponse(self._steps.metrics)

    @cached_property
    def trace(self) -> AsyncTraceResourceWithRawResponse:
        return AsyncTraceResourceWithRawResponse(self._steps.trace)

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithRawResponse:
        return AsyncFeedbackResourceWithRawResponse(self._steps.feedback)

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._steps.messages)


class StepsResourceWithStreamingResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.retrieve = to_streamed_response_wrapper(
            steps.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            steps.list,
        )

    @cached_property
    def metrics(self) -> MetricsResourceWithStreamingResponse:
        return MetricsResourceWithStreamingResponse(self._steps.metrics)

    @cached_property
    def trace(self) -> TraceResourceWithStreamingResponse:
        return TraceResourceWithStreamingResponse(self._steps.trace)

    @cached_property
    def feedback(self) -> FeedbackResourceWithStreamingResponse:
        return FeedbackResourceWithStreamingResponse(self._steps.feedback)

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._steps.messages)


class AsyncStepsResourceWithStreamingResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.retrieve = async_to_streamed_response_wrapper(
            steps.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            steps.list,
        )

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithStreamingResponse:
        return AsyncMetricsResourceWithStreamingResponse(self._steps.metrics)

    @cached_property
    def trace(self) -> AsyncTraceResourceWithStreamingResponse:
        return AsyncTraceResourceWithStreamingResponse(self._steps.trace)

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithStreamingResponse:
        return AsyncFeedbackResourceWithStreamingResponse(self._steps.feedback)

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._steps.messages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

from .sse_mcp_server import SseMcpServer
from .stdio_mcp_server import StdioMcpServer
from .streamable_http_mcp_server import StreamableHTTPMcpServer

__all__ = ["McpServerListResponse", "McpServerListResponseItem"]

McpServerListResponseItem: TypeAlias = Union[StdioMcpServer, SseMcpServer, StreamableHTTPMcpServer]

McpServerListResponse: TypeAlias = List[McpServerListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import archive_list_params, archive_create_params, archive_update_params
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .passages import (
    PassagesResource,
    AsyncPassagesResource,
    PassagesResourceWithRawResponse,
    AsyncPassagesResourceWithRawResponse,
    PassagesResourceWithStreamingResponse,
    AsyncPassagesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.archive import Archive
from ...types.embedding_config_param import EmbeddingConfigParam

__all__ = ["ArchivesResource", "AsyncArchivesResource"]


class ArchivesResource(SyncAPIResource):
    @cached_property
    def passages(self) -> PassagesResource:
        return PassagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ArchivesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return ArchivesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArchivesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return ArchivesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        description: Optional[str] | Omit = omit,
        embedding: Optional[str] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Create a new archive.

        Args:
          embedding: Embedding model handle for the archive

          embedding_config: Configuration for embedding model connection and processing parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/archives/",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "embedding": embedding,
                    "embedding_config": embedding_config,
                },
                archive_create_params.ArchiveCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    def retrieve(
        self,
        archive_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Get a single archive by its ID.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        return self._get(
            f"/v1/archives/{archive_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    def update(
        self,
        archive_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Update an existing archive's name and/or description.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        return self._patch(
            f"/v1/archives/{archive_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                archive_update_params.ArchiveUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Archive]:
        """
        Get a list of all archives for the current organization with optional filters
        and pagination.

        Args:
          after: Archive ID cursor for pagination. Returns archives that come after this archive
              ID in the specified sort order

          agent_id: Only archives attached to this agent ID

          before: Archive ID cursor for pagination. Returns archives that come before this archive
              ID in the specified sort order

          limit: Maximum number of archives to return

          name: Filter by archive name (exact match)

          order: Sort order for archives by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/archives/",
            page=SyncArrayPage[Archive],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "before": before,
                        "limit": limit,
                        "name": name,
                        "order": order,
                        "order_by": order_by,
                    },
                    archive_list_params.ArchiveListParams,
                ),
            ),
            model=Archive,
        )

    def delete(
        self,
        archive_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an archive by its ID.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/archives/{archive_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncArchivesResource(AsyncAPIResource):
    @cached_property
    def passages(self) -> AsyncPassagesResource:
        return AsyncPassagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncArchivesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncArchivesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArchivesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncArchivesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        description: Optional[str] | Omit = omit,
        embedding: Optional[str] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Create a new archive.

        Args:
          embedding: Embedding model handle for the archive

          embedding_config: Configuration for embedding model connection and processing parameters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/archives/",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "embedding": embedding,
                    "embedding_config": embedding_config,
                },
                archive_create_params.ArchiveCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    async def retrieve(
        self,
        archive_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Get a single archive by its ID.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        return await self._get(
            f"/v1/archives/{archive_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    async def update(
        self,
        archive_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Archive:
        """
        Update an existing archive's name and/or description.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        return await self._patch(
            f"/v1/archives/{archive_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                archive_update_params.ArchiveUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Archive,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        agent_id: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Archive, AsyncArrayPage[Archive]]:
        """
        Get a list of all archives for the current organization with optional filters
        and pagination.

        Args:
          after: Archive ID cursor for pagination. Returns archives that come after this archive
              ID in the specified sort order

          agent_id: Only archives attached to this agent ID

          before: Archive ID cursor for pagination. Returns archives that come before this archive
              ID in the specified sort order

          limit: Maximum number of archives to return

          name: Filter by archive name (exact match)

          order: Sort order for archives by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/archives/",
            page=AsyncArrayPage[Archive],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "agent_id": agent_id,
                        "before": before,
                        "limit": limit,
                        "name": name,
                        "order": order,
                        "order_by": order_by,
                    },
                    archive_list_params.ArchiveListParams,
                ),
            ),
            model=Archive,
        )

    async def delete(
        self,
        archive_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an archive by its ID.

        Args:
          archive_id: The ID of the archive in the format 'archive-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not archive_id:
            raise ValueError(f"Expected a non-empty value for `archive_id` but received {archive_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/archives/{archive_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ArchivesResourceWithRawResponse:
    def __init__(self, archives: ArchivesResource) -> None:
        self._archives = archives

        self.create = to_raw_response_wrapper(
            archives.create,
        )
        self.retrieve = to_raw_response_wrapper(
            archives.retrieve,
        )
        self.update = to_raw_response_wrapper(
            archives.update,
        )
        self.list = to_raw_response_wrapper(
            archives.list,
        )
        self.delete = to_raw_response_wrapper(
            archives.delete,
        )

    @cached_property
    def passages(self) -> PassagesResourceWithRawResponse:
        return PassagesResourceWithRawResponse(self._archives.passages)


class AsyncArchivesResourceWithRawResponse:
    def __init__(self, archives: AsyncArchivesResource) -> None:
        self._archives = archives

        self.create = async_to_raw_response_wrapper(
            archives.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            archives.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            archives.update,
        )
        self.list = async_to_raw_response_wrapper(
            archives.list,
        )
        self.delete = async_to_raw_response_wrapper(
            archives.delete,
        )

    @cached_property
    def passages(self) -> AsyncPassagesResourceWithRawResponse:
        return AsyncPassagesResourceWithRawResponse(self._archives.passages)


class ArchivesResourceWithStreamingResponse:
    def __init__(self, archives: ArchivesResource) -> None:
        self._archives = archives

        self.create = to_streamed_response_wrapper(
            archives.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            archives.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            archives.update,
        )
        self.list = to_streamed_response_wrapper(
            archives.list,
        )
        self.delete = to_streamed_response_wrapper(
            archives.delete,
        )

    @cached_property
    def passages(self) -> PassagesResourceWithStreamingResponse:
        return PassagesResourceWithStreamingResponse(self._archives.passages)


class AsyncArchivesResourceWithStreamingResponse:
    def __init__(self, archives: AsyncArchivesResource) -> None:
        self._archives = archives

        self.create = async_to_streamed_response_wrapper(
            archives.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            archives.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            archives.update,
        )
        self.list = async_to_streamed_response_wrapper(
            archives.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            archives.delete,
        )

    @cached_property
    def passages(self) -> AsyncPassagesResourceWithStreamingResponse:
        return AsyncPassagesResourceWithStreamingResponse(self._archives.passages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from letta_client import Letta, AsyncLetta
from letta_client.types import (
    McpServerListResponse,
    McpServerCreateResponse,
    McpServerUpdateResponse,
    McpServerRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMcpServers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.create(
            config={
                "args": ["string"],
                "command": "command",
                "env": {"foo": "string"},
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.retrieve(
            "mcp_server_id",
        )
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.retrieve(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.retrieve(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.mcp_servers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "env": {"foo": "string"},
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.mcp_servers.with_raw_response.update(
                mcp_server_id="",
                config={
                    "args": ["string"],
                    "command": "command",
                    "mcp_server_type": "stdio",
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.list()
        assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.delete(
            "mcp_server_id",
        )
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.delete(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.delete(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert mcp_server is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.mcp_servers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_refresh(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.refresh(
            mcp_server_id="mcp_server_id",
        )
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_refresh_with_all_params(self, client: Letta) -> None:
        mcp_server = client.mcp_servers.refresh(
            mcp_server_id="mcp_server_id",
            agent_id="agent_id",
        )
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_refresh(self, client: Letta) -> None:
        response = client.mcp_servers.with_raw_response.refresh(
            mcp_server_id="mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = response.parse()
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_refresh(self, client: Letta) -> None:
        with client.mcp_servers.with_streaming_response.refresh(
            mcp_server_id="mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = response.parse()
            assert_matches_type(object, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_refresh(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            client.mcp_servers.with_raw_response.refresh(
                mcp_server_id="",
            )


class TestAsyncMcpServers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.create(
            config={
                "args": ["string"],
                "command": "command",
                "env": {"foo": "string"},
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.create(
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerCreateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.retrieve(
            "mcp_server_id",
        )
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.retrieve(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.retrieve(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerRetrieveResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.mcp_servers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "env": {"foo": "string"},
                "mcp_server_type": "stdio",
            },
            server_name="server_name",
        )
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.update(
            mcp_server_id="mcp_server_id",
            config={
                "args": ["string"],
                "command": "command",
                "mcp_server_type": "stdio",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerUpdateResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.mcp_servers.with_raw_response.update(
                mcp_server_id="",
                config={
                    "args": ["string"],
                    "command": "command",
                    "mcp_server_type": "stdio",
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.list()
        assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(McpServerListResponse, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.delete(
            "mcp_server_id",
        )
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.delete(
            "mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert mcp_server is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.delete(
            "mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert mcp_server is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.mcp_servers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_refresh(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.refresh(
            mcp_server_id="mcp_server_id",
        )
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_refresh_with_all_params(self, async_client: AsyncLetta) -> None:
        mcp_server = await async_client.mcp_servers.refresh(
            mcp_server_id="mcp_server_id",
            agent_id="agent_id",
        )
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_refresh(self, async_client: AsyncLetta) -> None:
        response = await async_client.mcp_servers.with_raw_response.refresh(
            mcp_server_id="mcp_server_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mcp_server = await response.parse()
        assert_matches_type(object, mcp_server, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_refresh(self, async_client: AsyncLetta) -> None:
        async with async_client.mcp_servers.with_streaming_response.refresh(
            mcp_server_id="mcp_server_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mcp_server = await response.parse()
            assert_matches_type(object, mcp_server, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_refresh(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcp_server_id` but received ''"):
            await async_client.mcp_servers.with_raw_response.refresh(
                mcp_server_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.step import Step
from ...types.steps import feedback_create_params
from ..._base_client import make_request_options

__all__ = ["FeedbackResource", "AsyncFeedbackResource"]


class FeedbackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return FeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return FeedbackResourceWithStreamingResponse(self)

    def create(
        self,
        step_id: str,
        *,
        feedback: Optional[Literal["positive", "negative"]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Modify feedback for a given step.

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          feedback: Whether this feedback is positive or negative

          tags: Feedback tags to add to the step

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._patch(
            f"/v1/steps/{step_id}/feedback",
            body=maybe_transform(
                {
                    "feedback": feedback,
                    "tags": tags,
                },
                feedback_create_params.FeedbackCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )


class AsyncFeedbackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncFeedbackResourceWithStreamingResponse(self)

    async def create(
        self,
        step_id: str,
        *,
        feedback: Optional[Literal["positive", "negative"]] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Modify feedback for a given step.

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          feedback: Whether this feedback is positive or negative

          tags: Feedback tags to add to the step

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._patch(
            f"/v1/steps/{step_id}/feedback",
            body=await async_maybe_transform(
                {
                    "feedback": feedback,
                    "tags": tags,
                },
                feedback_create_params.FeedbackCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )


class FeedbackResourceWithRawResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.create = to_raw_response_wrapper(
            feedback.create,
        )


class AsyncFeedbackResourceWithRawResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.create = async_to_raw_response_wrapper(
            feedback.create,
        )


class FeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.create = to_streamed_response_wrapper(
            feedback.create,
        )


class AsyncFeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.create = async_to_streamed_response_wrapper(
            feedback.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["ConversationListParams"]


class ConversationListParams(TypedDict, total=False):
    after: Optional[str]
    """Cursor for pagination (conversation ID)"""

    agent_id: Optional[str]
    """
    The agent ID to list conversations for (optional - returns all conversations if
    not provided)
    """

    limit: int
    """Maximum number of conversations to return"""

    order: Literal["asc", "desc"]
    """Sort order for conversations. 'asc' for oldest first, 'desc' for newest first"""

    order_by: Literal["created_at", "last_run_completion"]
    """Field to sort by"""

    summary_search: Optional[str]
    """Search for text within conversation summaries"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ReasoningMessage"]


class ReasoningMessage(BaseModel):
    """Representation of an agent's internal reasoning.

    Args:
        id (str): The ID of the message
        date (datetime): The date the message was created in ISO format
        name (Optional[str]): The name of the sender of the message
        source (Literal["reasoner_model", "non_reasoner_model"]): Whether the reasoning
            content was generated natively by a reasoner model or derived via prompting
        reasoning (str): The internal reasoning of the agent
        signature (Optional[str]): The model-generated signature of the reasoning step
    """

    id: str

    date: datetime

    reasoning: str

    is_err: Optional[bool] = None

    message_type: Optional[Literal["reasoning_message"]] = None
    """The type of the message."""

    name: Optional[str] = None

    otid: Optional[str] = None

    run_id: Optional[str] = None

    sender_id: Optional[str] = None

    seq_id: Optional[int] = None

    signature: Optional[str] = None

    source: Optional[Literal["reasoner_model", "non_reasoner_model"]] = None

    step_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .stop_reason_type import StopReasonType
from .agents.internal_message import InternalMessage

__all__ = ["Step"]


class Step(BaseModel):
    id: str
    """The id of the step. Assigned by the database."""

    agent_id: Optional[str] = None
    """The ID of the agent that performed the step."""

    cache_write_tokens: Optional[int] = None
    """The number of input tokens written to cache (Anthropic only).

    None if not reported by provider.
    """

    cached_input_tokens: Optional[int] = None
    """The number of input tokens served from cache. None if not reported by provider."""

    completion_tokens: Optional[int] = None
    """The number of tokens generated by the agent during this step."""

    completion_tokens_details: Optional[Dict[str, object]] = None
    """Detailed completion token breakdown (e.g., reasoning_tokens)."""

    context_window_limit: Optional[int] = None
    """The context window limit configured for this step."""

    error_data: Optional[Dict[str, object]] = None
    """Error details including message, traceback, and additional context"""

    error_type: Optional[str] = None
    """The type/class of the error that occurred"""

    feedback: Optional[Literal["positive", "negative"]] = None
    """The feedback for this step. Must be either 'positive' or 'negative'."""

    messages: Optional[List[InternalMessage]] = None
    """The messages generated during this step.

    Deprecated: use `GET /v1/steps/{step_id}/messages` endpoint instead
    """

    model: Optional[str] = None
    """The name of the model used for this step."""

    api_model_endpoint: Optional[str] = FieldInfo(alias="model_endpoint", default=None)
    """The model endpoint url used for this step."""

    api_model_handle: Optional[str] = FieldInfo(alias="model_handle", default=None)
    """The model handle (e.g., 'openai/gpt-4o-mini') used for this step."""

    origin: Optional[str] = None
    """The surface that this agent step was initiated from."""

    project_id: Optional[str] = None
    """The project that the agent that executed this step belongs to (cloud only)."""

    prompt_tokens: Optional[int] = None
    """The number of tokens in the prompt during this step."""

    prompt_tokens_details: Optional[Dict[str, object]] = None
    """
    Detailed prompt token breakdown (e.g., cached_tokens, cache_read_tokens,
    cache_creation_tokens).
    """

    provider_category: Optional[str] = None
    """The category of the provider used for this step."""

    provider_id: Optional[str] = None
    """The unique identifier of the provider that was configured for this step"""

    provider_name: Optional[str] = None
    """The name of the provider used for this step."""

    reasoning_tokens: Optional[int] = None
    """The number of reasoning/thinking tokens generated.

    None if not reported by provider.
    """

    request_id: Optional[str] = None
    """The API request log ID from cloud-api for correlating steps with API requests."""

    run_id: Optional[str] = None
    """The unique identifier of the run that this step belongs to.

    Only included for async calls.
    """

    status: Optional[Literal["pending", "success", "failed", "cancelled"]] = None
    """Status of a step execution"""

    stop_reason: Optional[StopReasonType] = None
    """The stop reason associated with the step."""

    tags: Optional[List[str]] = None
    """Metadata tags."""

    tid: Optional[str] = None
    """The unique identifier of the transaction that processed this step."""

    total_tokens: Optional[int] = None
    """The total number of tokens processed by the agent during this step."""

    trace_id: Optional[str] = None
    """The trace id of the agent step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List
from typing_extensions import TypeAlias

__all__ = ["TraceRetrieveResponse"]

TraceRetrieveResponse: TypeAlias = List[Dict[str, object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["BlockUpdateParams"]


class BlockUpdateParams(TypedDict, total=False):
    base_template_id: Optional[str]
    """The base template id of the block."""

    deployment_id: Optional[str]
    """The id of the deployment."""

    description: Optional[str]
    """Description of the block."""

    entity_id: Optional[str]
    """The id of the entity within the template."""

    hidden: Optional[bool]
    """If set to True, the block will be hidden."""

    is_template: bool
    """Whether the block is a template (e.g. saved human/persona options)."""

    label: Optional[str]
    """Label of the block (e.g. 'human', 'persona') in the context window."""

    limit: Optional[int]
    """Character limit of the block."""

    metadata: Optional[Dict[str, object]]
    """Metadata of the block."""

    preserve_on_migration: Optional[bool]
    """Preserve the block on template migration."""

    project_id: Optional[str]
    """The associated project id."""

    read_only: bool
    """Whether the agent has read-only access to the block."""

    tags: Optional[SequenceNotStr[str]]
    """The tags to associate with the block."""

    template_id: Optional[str]
    """The id of the template."""

    template_name: Optional[str]
    """Name of the block if it is a template."""

    value: Optional[str]
    """Value of the block."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ToolCall"]


class ToolCall(BaseModel):
    arguments: str

    name: str

    tool_call_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = ["ImageContent", "Source", "SourceURLImage", "SourceBase64Image", "SourceLettaImage"]


class SourceURLImage(BaseModel):
    url: str
    """The URL of the image."""

    type: Optional[Literal["url"]] = None
    """The source type for the image."""


class SourceBase64Image(BaseModel):
    data: str
    """The base64 encoded image data."""

    media_type: str
    """The media type for the image."""

    detail: Optional[str] = None
    """
    What level of detail to use when processing and understanding the image (low,
    high, or auto to let the model decide)
    """

    type: Optional[Literal["base64"]] = None
    """The source type for the image."""


class SourceLettaImage(BaseModel):
    file_id: str
    """The unique identifier of the image file persisted in storage."""

    data: Optional[str] = None
    """The base64 encoded image data."""

    detail: Optional[str] = None
    """
    What level of detail to use when processing and understanding the image (low,
    high, or auto to let the model decide)
    """

    media_type: Optional[str] = None
    """The media type for the image."""

    type: Optional[Literal["letta"]] = None
    """The source type for the image."""


Source: TypeAlias = Annotated[
    Union[SourceURLImage, SourceBase64Image, SourceLettaImage], PropertyInfo(discriminator="type")
]


class ImageContent(BaseModel):
    source: Source
    """The source of the image."""

    type: Optional[Literal["image"]] = None
    """The type of the message."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ParentToolRule"]


class ParentToolRule(BaseModel):
    """
    A ToolRule that only allows a child tool to be called if the parent has been called.
    """

    children: List[str]
    """The children tools that can be invoked."""

    tool_name: str
    """The name of the tool. Must exist in the database for the user's organization."""

    prompt_template: Optional[str] = None
    """Optional template string (ignored)."""

    type: Optional[Literal["parent_last_tool"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .text_content import TextContent
from .image_content import ImageContent

__all__ = ["ToolReturn", "ToolReturnUnionMember0"]

ToolReturnUnionMember0: TypeAlias = Annotated[Union[TextContent, ImageContent], PropertyInfo(discriminator="type")]


class ToolReturn(BaseModel):
    status: Literal["success", "error"]

    tool_call_id: str

    tool_return: Union[List[ToolReturnUnionMember0], str]
    """The tool return value - either a string or list of content parts (text/image)"""

    stderr: Optional[List[str]] = None

    stdout: Optional[List[str]] = None

    type: Optional[Literal["tool"]] = None
    """The message type to be created."""
